/*
	File name:	arithm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		01/01/97
	Function:	Doing some arithmetic operation on PGM images.
*/
#include <stdio.h>
#include <stdlib.h>
#include "must.h"
#include "pgm.h"

#define	GRAY_LEVELS	256
long	hist[GRAY_LEVELS];

void	usage(FILE *fp, char *cmd);
void	get_hist(uchar buf[], long size, long hist[], int bins);

int	main(int argc, char *argv[])
{
	int	width1, height1;
	uchar	**image1;
	int	width2, height2;
	uchar	**image2;

	int	width, height;
	int	i, j;
	char	sch;

	char	op;
	int	num_input, has_output;
	char	*inname1, *inname2, *outname;

	if(argc<3) {
		usage(stderr, argv[0]);
		return -1;
	}

	op=argv[1][0];
	switch(op) {
		case '+':
		case '-':
		case '&':
		case '|':
		case '@':
		case '^':
		case '\\':
		case 'm':
		case 'r':
			num_input = 2;
			has_output= 1;
			break;
		case 'a':
		case 'b':
		case 'i':
		case 's':
		case '4':
		case 't':
			num_input = 1;
			has_output= 1;
			break;
		case 'h':
			num_input = 1;
			has_output= 0;
			break;
		default:
			fprintf(stderr, "invalid operation :%c\n", op);
			usage(stderr, argv[0]);
			return -2;
	}

	if(argc<2+num_input+has_output) {
		usage(stderr, argv[0]);
		return -1;
	}

	inname1 = argv[2];
	inname2 = argv[1+num_input];
	if(has_output)
		outname = argv[2+num_input];

	i=ReadPGM(inname1, &width1, &height1, &image1);
	if(num_input>=2)
		j=ReadPGM(inname2, &width2, &height2, &image2);
	else {
		j	= 0;
		inname2	= NULL;
		width2	= width1;
		height2	= height1;
		image2	= NULL;
	}

	if(i || j) {
		fprintf(stderr, "Error openning input image(s).\n");
		if(image1!=NULL) free2d((void**)image1);
		if(image2!=NULL) free2d((void**)image2);
		return -3;
	}

	width	= (width1<=width2 ? width1 : width2);
	height	= (height1<=height2 ? height1 : height2);

	switch(op) {
	case '+':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				image1[i][j] += image2[i][j];
		break;
	case '-':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				image1[i][j] -= image2[i][j];
		break;
	case '&':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j] && image2[i][j])
					image1[i][j] = 255;
				else
					image1[i][j] = 0;
		break;
	case '|':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j] || image2[i][j])
					image1[i][j] = 255;
		break;
	case '@':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j] == image2[i][j])
					image1[i][j] = 255;
				else
					image1[i][j] = 0;
		break;
	case '^':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j] != image2[i][j])
					image1[i][j] = 255;
				else
					image1[i][j] = 0;
		break;
	case '\\':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j] && !image2[i][j])
					image1[i][j] = 255;
				else
					image1[i][j] = 0;
		break;
	case 'm':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(!image2[i][j]) image1[i][j] = 0;
		break;
	case 'r':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image2[i][j]) image1[i][j] = image2[i][j];
		break;
	case 'a':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++) {
				sch=image1[i][j];
				if(sch<0) sch=-sch;
				image1[i][j]=sch;
			}
		break;
	case 'b':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j])
					image1[i][j] = 255;
		break;
	case 'i':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				image1[i][j] = 255 - image1[i][j];
		break;
	case 's':
		{
		uchar	val, minval=255, maxval=0;
		float	ratio;

		for(i=0;i<height;i++)
			for(j=0;j<width;j++) {
				val=image1[i][j];
				if(val < minval) minval=val;
				if(val > maxval) maxval=val;
			}
		if(minval==maxval) {
			for(i=0;i<height;i++)
				for(j=0;j<width;j++)
					image1[i][j]=255;
			break;
		}
		ratio = 255.0 / (maxval-minval);
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				image1[i][j] = (image1[i][j]-minval)*ratio;
		break;
		}
	case '4':
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j]==255) image1[i][j]=254;
		break;
	case 'h':
		printf("%d\n", GRAY_LEVELS);
		get_hist(image1[0], (long)width*height, hist, GRAY_LEVELS);
		for(i=0;i<GRAY_LEVELS;i++)
			printf("%d %ld\n", i, hist[i]);
		break;
	case 't':
		{
		int	T;

		get_hist(image1[0], (long)width*height, hist, GRAY_LEVELS);
		i=otsu(hist, GRAY_LEVELS, &T);
		if(i) {
			fprintf(stderr, "otsu() failed with status %d\n", i);
			exit(-4);
		}
		for(i=0;i<height;i++)
			for(j=0;j<width;j++)
				if(image1[i][j]>=T)	image1[i][j]=255;
				else			image1[i][j]=0;
		break;
		}
	}

	if(has_output) {
		i=WritePGM(outname, width, height, image1);
		if(i) fprintf(stderr, "Error writing output.\n");
	}

	free2d((void**)image1);
	if(num_input>=2) free2d((void**)image2);

	return 0;
}

void	get_hist(uchar buf[], long size, long hist[], int bins)
{
	long	k;
	for(k=0;k<bins;k++) hist[k]=0;
	for(k=0;k<size;k++) hist[ buf[k] ] ++;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp, "Usage:\n\t %s <+-&|@^\\mabirs4ht> <PGM1> [PGM2] [PGM3]\n", cmd);
fprintf(fp, "Options:\n");
fprintf(fp, "\t +   --- arithmetic plus\n");
fprintf(fp, "\t -   --- arithmetic minus\n");
fprintf(fp, "\t &   --- logical and\n");
fprintf(fp, "\t |   --- logical or\n");
fprintf(fp, "\t @   --- logical ior\n");
fprintf(fp, "\t ^   --- logical xor\n");
fprintf(fp, "\t \\   --- logical subtract\n");
fprintf(fp, "\t m   --- masking input file 1 by input file 2\n");
fprintf(fp, "\t a   --- take the absolute value (signed input image)\n");
fprintf(fp, "\t b   --- convert to binary (zero -->0, non-zero -->255)\n");
fprintf(fp, "\t i   --- invert (255 minus)\n");
fprintf(fp, "\t r   --- replace pixels in PGM1 with non-zero's in PGM2\n");
fprintf(fp, "\t s   --- stretch (to 0 to 255)\n");
fprintf(fp, "\t 4   --- change pixel of 255 to 254.\n");
fprintf(fp, "\t h   --- get the histogram of the image.\n");
fprintf(fp, "\t t   --- threshold the image using Otsu's method.\n");
fprintf(fp, "\n");
fprintf(fp, "<PGM1> is input, [PGM3] is output.\n");
fprintf(fp, "[PGM2] is \t input for +-&|@^\\mr;\n");
fprintf(fp, "\t\t void for h;\n");
fprintf(fp, "\t\t output for others.\n");
fprintf(fp, "\n");
return;
}

