/*
	File name:	editpgm2.c
	Language:	C
	Author:		Gang Liu
	Date:		10/19/97
	Type:		main program
	Function:	Given an image and a series of triples of (x,y,v),
			change the pixel value at (x,y) to v.

			Modified from editpgm.c to read the triples from a
			disk file, since shell might not be able to handle a
			very long list of triples.

*/
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>

#include "must.h"
#include "pgm.h"

void	usage(FILE *fp, char *cmd);

int	main(int argc, char *argv[])
{
	int	i;

	char	*inname, *txtname, *outname;

	FILE	*fp;
	uchar	**image;
	int	width, height;

	int	x, y, v;

	if(argc != 4) {
		usage(stderr, argv[0]);
		return -1;
	}

	i=1;
	inname = argv[i++];
	txtname = argv[i++];
	outname = argv[i++];

	i=ReadPGM(inname, &width, &height, &image);
	if(i) myerror(i, stderr, "Error reading input image.\n");

	fp=fopen(txtname, "rt");
	if(fp==NULL) myerror(-1, stderr, "Error openning text file.\n");

	for(;;) {
		i=fscanf(fp, "%d %d %d", &x, &y, &v);
		if(feof(fp)) break;
		if(i!=3) myerror(i, stderr, "Error reading text file.\n");

		if(x<0 || x>=width || y<0 || y>=height) {
			fprintf(stderr,
			"(x,y)=(%d,%d) out of image, ignored.\n", x, y);
			continue;
		}
		if(v<0) v=0;
		if(v>255) v=255;

		image[y][x]=v;
	}
	fclose(fp);

	i=WritePGM(outname, width, height, image);
	if(i) myerror(i, stderr, "Error writing output image.\n");

	free2d((void**)image);

	return 0;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp,
"Usage: %s <PGM1> <txt> <PGM2>\n\
where\n\
\t <PGM1> is the input image file (PGM format) name\n\
\t <txt>  is the plain text file containing the \"x y v\" triples\n\
\t <PGM2> is the output image file (PGM format) name\n\
\n", cmd);
}
