/*
	File name:	histpgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		05/22/97
	Function:	Read in an image in a PGM file and sends its histogram
			in two-column text format to stdout.
*/
#include <stdio.h>

#include "must.h"
#include "pgm.h"

#define	GRAY_LEVELS	256

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	i, j;
	int	bins=GRAY_LEVELS;
	long	hist[GRAY_LEVELS];

	char	*dataname, *maskname;

	uchar	**image, **mask;
	int	width, height;
	int	wi, hi, wm, hm;


	if(argc<2) {
		usage(argv[0]);
		return -1;
	}


	dataname=argv[1];
	maskname=NULL;
	if(argc>=3) maskname=argv[2];

	i=0;
	i|=ReadPGM(dataname, &wi, &hi, &image);
	if(maskname!=NULL)
		i|=ReadPGM(maskname, &wm, &hm, &mask);
	if(i) {
		fprintf(stderr, "Error reading input images.\n");
		return -2;
	}

	if(maskname==NULL) {
		wm=wi; hm=hi;
	}

	if(wm!=wi || hm!=hi) 
		fprintf(stderr, "Warning: two images of different size.\n");

	width=wi;	height=hi;
	if(wm<width) width=wm;
	if(hm<height) height=hm;

	for(i=0;i<bins;i++) hist[i]=0;

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			if(maskname!=NULL)
				if(mask[i][j]==0) continue;
			hist[ image[i][j] ] ++;
		}

	fprintf(stdout, "%d\n", bins);
	for(i=0;i<bins;i++)
		fprintf(stdout, "%d %ld\n", i, hist[i]);

	free2d((void**)image);
	if(maskname!=NULL) free2d((void**)mask);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <PGM1> [PGM2]\n", cmd);
fprintf(stderr, "<PGM1> is the image whose histogram is to be obtained;\n");
fprintf(stderr, "[PGM2] is the optional mask image file;\n");
fprintf(stderr, "both files should be in PGM format.\n");
fprintf(stderr, "\n");
}
