/*
	File name:	makepgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		06/30/97
	Function:	Make a PGM file of an image with a specified constant
			pixel value.
*/
#include <stdio.h>
#include <stdlib.h>

#include "must.h"
#include "pgm.h"

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	i, j;

	int	width, height;
	uchar	**image;

	char	*fname;
	uchar	val;

	if(argc<5) {
		usage(argv[0]);
		return -1;
	}

	width=atoi(argv[1]);
	height=atoi(argv[2]);
	val=atoi(argv[3]);
	if(width<=0 || height<=0)
		myerror(-1, stderr, "Wrong width or height.\n");

	fname=argv[4];

	image=(uchar**)alloc2d(width, height, sizeof(char));
	if(image==NULL)
		myerror(-2, stderr, "Error allocating memory.\n");

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) image[i][j]=val;

	i=WritePGM(fname, width, height, image);
	if(i) myerror(-3, stderr, "Error writing output file.\n");

	free2d((void**)image);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <width> <height> <val> <name>\n", cmd);
fprintf(stderr, "<width> is the width of the image.\n");
fprintf(stderr, "<height> is the width of the image.\n");
fprintf(stderr, "<val> is the constant pixel value of the image.\n");
fprintf(stderr, "<name> is the name of the PGM file to be created.\n");
fprintf(stderr, "\n");
return;
}
