/*
	File name:	outerbound.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		11/16/99
	Function:	Follow the outer boundary of the first encountered
			connected component which contains the specified
			pixel location.
*/
#include <stdio.h>
#include <stdlib.h>

#include "must.h"
#include "pgm.h"
#include "boundary.h"

void	usage(FILE *fp, char *cmd);
void	get_boundary(uchar **areamap, int width, int height, uchar **boundmap);

int	main(int argc, char *argv[])
{
	char	*inname, *outname;
	int	val, conn, verbose;

	int	width, height;
	uchar	**image;
	ChainCode	bound;

	int	i, j;

	if(argc<6) {
		usage(stderr, argv[0]);
		return -1;
	}

	i=1;
	inname=argv[i++];
	val=atoi(argv[i++]);
	conn=atoi(argv[i++]);
	outname=argv[i++];

	verbose=0;
	if(argc>=6 && toupper(argv[5][0])=='V') verbose=1;

	if(conn!=4 && conn!=8) conn=8;
	if(val<0) val=-val;
	val%=256;

	i=ReadPGM(inname, &width, &height, &image);
	if(i) myerror(-2, stderr, "Error reading input image.\n");

	i=find_boundary(image, width, height, val, conn, &bound);
	if(i) myerror(i, stderr, "find_boundary() failed.\n");

	if(verbose) {
		int	len;
		point	*list;
		i=chain2list(&bound, &list, &len);
		if(i) myerror(i, stderr, "chain2list() failed.");
		for(i=0;i<len;i++) printf("%d %d\n", list[i].x, list[i].y);
		free(list);
	}

	for(i=0;i<height;i++)
		for(j=0;j<width;j++) image[i][j]=0;

	draw_boundary(image, width, height, 255, &bound);

	i=WritePGM(outname, width, height, image);
	if(i) myerror(-4, stderr, "Error writing output.\n");

	free_chain(&bound);
	free2d((void**)image);

	return 0;
}

void	usage(FILE *fp, char *cmd)
{
fprintf(fp,
"Usage:\n\t %s <PGM1> <val> <8|4> <PGM2> [v]\n\
where\n\
	<PGM1>	is the input PGM file name\n\
	<val>	is the pixel value to search for\n\
	<8|4>	is the connectivity to use, either 8 or 4\n\
	<PGM2>	is the output PGM file name\n\
	[v]	verbose.\n\
\n", cmd);
return;
}

