/*
	File name:	pgms2ppm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		06/30/97
	Function:	Create a RGB image in PPM format from 3 PGM images
			taken as the red, green, and blue band data.
*/
#include <stdio.h>

#include "must.h"
#include "pgm.h"
#include "ppm.h"

#ifndef	mymin
#define	mymin(x,y)	((x)<=(y)?(x):(y))
#endif

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	width, height;
	int	wred, hred, wgreen, hgreen, wblue, hblue;
	uchar	**red, **green, **blue;

	char	*redname, *greenname, *bluename, *outname;
	FILE	*fp;

	int	i, j;

	uchar	c1r, c1g, c1b, c2r, c2g, c2b, err;

	if(argc<5) {
		usage(argv[0]);
		return -1;
	}

	redname=argv[1];
	greenname=argv[2];
	bluename=argv[3];
	outname=argv[4];

	i=0;
	i|=ReadPGM(redname, &wred, &hred, &red);
	i|=ReadPGM(greenname, &wgreen, &hgreen, &green);
	i|=ReadPGM(bluename, &wblue, &hblue, &blue);
	if(i) myerror(-2, stderr, "Error reading input images.\n");

	width=mymin(mymin(wred, wgreen), wblue);
	height=mymin(mymin(hred, hgreen), hblue);

	if(wred!=wgreen || wgreen!=wblue || hred!=hgreen || hgreen!=hblue)
		fprintf(stderr, "Warning: input image sizes are different.\n");

	fp=fopen(outname, "w");
	if(fp==NULL) myerror(-3, stderr, "Error openning output file.\n");

	fprintf(fp, "P6\n");
	fprintf(fp, "%d %d\n", width, height);
	fprintf(fp, "255\n");

	err=0;
	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			c1r=red[i][j];
			c1g=green[i][j];
			c1b=blue[i][j];
			c2r=fputc(c1r, fp);
			c2g=fputc(c1g, fp);
			c2b=fputc(c1b, fp);
			if(c1r!=c2r || c1g!=c2g || c1b!=c2b) err=1;
		}
	err |= fclose(fp);
	if(err) myerror(-4, stderr, "Error writing output data.\n");

	free2d((void**)red);
	free2d((void**)green);
	free2d((void**)blue);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <PGM red> <PGM green> <PGM blue> <PPM>\n", cmd);
fprintf(stderr, "<PGM red> is the name of the RED band PGM file.\n");
fprintf(stderr, "<PGM green> is the name of the GREEN band PGM file.\n");
fprintf(stderr, "<PGM blue> is the name of the BLUE band PGM file.\n");
fprintf(stderr, "<PPM> is the name of the output PPM file.\n");
fprintf(stderr, "\n");
return;
}
