#ifndef	_PPM_C_
#define _PPM_C_
#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "ppm.h"

typedef	unsigned char uchar;

int	ReadPPM(char *name, int *width, int *height, uchar ***(*data) );
int	WritePPM(char *name, int width, int height, uchar ***data);

void	***alloc3d(int d1, int d2, int d3, int unitsize);
void	free3d(void ***p, int d1, int d2, int d3);

int	ReadPPM(char *name, int *width, int *height, uchar ***(*data) )
{
	int	i, j, k;
	uchar	***dp;
	char	cbuf[256];
	FILE	*fp;

	uchar	c, err;

	fp=fopen(name, "r");
	if(fp==NULL) return -1;

	fgets(cbuf, 256, fp);	/* first line, should be "P6\n" for PPM */
	if(strcmp(cbuf,"P6\n")) {
		fclose(fp);
		return -2;
	}

	for(;;) {
		fgets(cbuf, 256, fp);
		if(cbuf[0]!='#') break; /* comments */
	}

	i=sscanf(cbuf, "%d %d", width, height);
	if(i!=2) {
		fclose(fp);
		return -3;
	}
	fgets(cbuf, 256, fp);	/* 255 */

	dp=(uchar ***)alloc3d(PPM_BANDS, (*height), (*width), sizeof(uchar));
	if(dp==NULL) {
		fclose(fp);
		return -4;
	}

	err=0;
	for(i=0;i<*height;i++)
		for(j=0;j<*width;j++)
			for(k=0;k<PPM_BANDS;k++) {
				c=fgetc(fp);
				if(feof(fp)) {
					err=1;
					i=*height; j=*width; break;
				}
				dp[k][i][j]=c;
			}
	fclose(fp);

	if(err) {
		free3d((void***)dp, PPM_BANDS, *height, *width);
		return -5;
	}

	(*data)=dp;
	return 0;
}

int	WritePPM(char *name, int width, int height, uchar ***data)
{
	int	i, j, k;
	FILE	*fp;

	uchar	c1, c2, err;

	fp=fopen(name, "w");
	if(fp==NULL) return -1;

	fprintf(fp, "P6\n");	/* magic number for PPM files */
	/* maybe should put some comment lines here */
	fprintf(fp, "%d %d\n", width, height);
	fprintf(fp, "255\n");	/* 255 */

	err=0;
	for(i=0;i<height;i++)
		for(j=0;j<width;j++)
			for(k=0;k<PPM_BANDS;k++) {
				c1=data[k][i][j];
				c2=fputc(c1, fp);
				if(c1!=c2) err=1;
			}
	err |= fclose(fp);
	if(err) return -2;
	return 0;
}

void	***alloc3d(int d1, int d2, int d3, int unitsize)
{
	int	i, j;
	char	***p, *databuf;
	long	size1, size2, size3;

	if(d1<=0 || d2<=0 || d3<=0) return NULL;

	size1 = (long)d1*sizeof(char**);
	size2 = (long)d1*d2*sizeof(char*);
	size3 = (long)d1*d2*d3*unitsize;

	i=sizeof(double);
	if(size1%i) size1=((size1+i)/i)*i;
	if(size2%i) size2=((size2+i)/i)*i;

#define	SINGLE_BLOCK 0
#if SINGLE_BLOCK

	p=(char***)malloc(size1 + size2 + size3);
	if(p==NULL) return NULL;

	for(i=0;i<d1;i++) {
		p[i]=(char**)((char*)p + size1 + i*d2*sizeof(char*));
		for(j=0;j<d2;j++)
			p[i][j]=(char*)((char*)p + size1 + size2
					+ i*d2*d3*unitsize + j*d3*unitsize);
	}
#else

	p=(char***)malloc(size1 + size2);
	databuf=(char*)malloc(size3);
	if(p==NULL || databuf==NULL) {
		if(p!=NULL) free(p);
		if(databuf!=NULL) free(databuf);
		return NULL;
	}

	for(i=0;i<d1;i++) {
		p[i]=(char**)((char*)p + size1 + i*d2*sizeof(char*));
		for(j=0;j<d2;j++)
			p[i][j]=databuf + i*d2*d3*unitsize + j*d3*unitsize;
	}

#endif

	return ((void***)p);
}

void	free3d(void ***p, int d1, int d2, int d3)
{
	if(p==NULL) return;

#if ! SINGLE_BLOCK
	free(p[0][0]);
#endif
	free(p);
}

#endif
