/*
	File name:	scatterpgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		07/27/97
	Function:	Cut from an image in PGM format into a series of
			rectangular non-overlapping sub-images, each of which
			is to be saved in a seperate PGM file.
*/
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "must.h"
#include "pgm.h"

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	width1, height1;
	uchar	**image;

	int	width2, height2;
	int	thiswidth, thisheight;

	int	i, j, k;

	int	vblocks, hblocks, m, n, x1, y1, x2, y2;

	char	*inname, *outnamebase, outname[512];
	FILE	*fp;
	time_t	now;

	if(argc<5) {
		usage(argv[0]);
		return -1;
	}

	inname=argv[1];
	i =sscanf(argv[2], "%d", &width2);
	i+=sscanf(argv[3], "%d", &height2);
	outnamebase=argv[4];
	if(i!=2) myerror(-2, stderr, "Error reading corner locations.\n");

	i=ReadPGM(inname, &width1, &height1, &image);
	if(i) myerror(-3, stderr, "Error reading input image.\n");

	if(width2>width1) width2=width1;
	if(height2>height1) height2=height1;

	if(width2<=0 || height2<=0)
		myerror(-4, stderr, "Invalid size specification.\n");

	hblocks=(width1 + (width2-1)) / width2;
	vblocks=(height1 + (height2-1)) / height2;

	time(&now);

	for(m=0;m<vblocks;m++)
	    for(n=0;n<hblocks;n++) {

		x1=n*width2;
		y1=m*height2;
		x2=x1+width2-1;
		y2=y1+height2-1;
		if(x2>=width1) x2=width1-1;
		if(y2>=height1) y2=height1-1;
		thiswidth=x2-x1+1;
		thisheight=y2-y1+1;

		sprintf(outname, "%s_%02d_%02d.pgm", outnamebase, m+1, n+1);
		fp=fopen(outname, "w");
		if(fp==NULL) {
			fprintf(stderr, "Output file name:%s\n", outname);
			myerror(-5, stderr, "Error openning output file.\n");
		}

		fprintf(fp, "P5\n");
		/* begin comments */
		fprintf(fp, "# Name: %s\n", outname);
		fprintf(fp, "# Produced by scatterpgm on %s", ctime(&now));
		fprintf(fp, "# Cropped from %s\n", inname);
		fprintf(fp, "#   by taking the (%dx%d) region at (%d,%d)\n",
				thiswidth, thisheight, x1, y1);
		/* end comments */
		fprintf(fp, "%d %d\n", thiswidth, thisheight);
		fprintf(fp, "255\n");

		k=0;
		for(i=y1;i<=y2;i++) {
			j=fwrite(&(image[i][x1]), thiswidth, 1, fp);
			if(j!=1) {
				k=1;
				break;
			}
		}
		k|=fclose(fp);
		if(k) {
			fprintf(stderr, "For %s:\n", outname);
			myerror(-6, stderr, "Error writing output.\n");
		}
	}

	free2d((void**)image);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "Function of %s:\n", cmd);
fprintf(stderr,
"Given an input image in PGM format, this utility cuts it into a number of\n\
rectangular, mutually non-overlapping, sub-images of specified size. The\n\
sub-images will be saved in PGM format, with their locations in the original\n\
image saved in plain text in the comment part of the PGM files. The names\n\
of the output files will be determined by appending a serial number and\n\
the .pgm suffix to the string provided by the user.\n");
fprintf(stderr, "\n");
fprintf(stderr, "Usage:\n\t %s <PGM> <width> <height> <basename>\n", cmd);
fprintf(stderr, "<PGM> is the input image to be cut to pieces;\n");
fprintf(stderr, "<width> is the width of the sub-images;\n");
fprintf(stderr, "<width> is the height of the sub-images;\n");
fprintf(stderr, "<basename> is the common part of the output file names.\n");
fprintf(stderr, "\n");
}

