/*
	File name:	subppm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		05/08/97
	Function:	Cut a sub-image from a PPM file and save the result
			also in a PPM file.
*/
#include <stdio.h>
#include <time.h>

#include "must.h"
#include "ppm.h"

#ifndef	mymin
#define	mymin(x,y)	((x)<=(y)?(x):(y))
#define	mymax(x,y)	((x)>=(y)?(x):(y))
#endif

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	int	width1, height1;
	uchar	***image1;

	int	x1, y1, x2, y2;
	int	width2, height2;
	uchar	***image2;

	int	i, j, k;

	char	*inname, *outname;
	FILE	*fp;
	time_t	now;

	uchar	c1, c2, err;

	if(argc<7) {
		usage(argv[0]);

		return -1;
	}

	inname=argv[1];
	i =sscanf(argv[2], "%d", &x1);
	i+=sscanf(argv[3], "%d", &y1);
	i+=sscanf(argv[4], "%d", &x2);
	i+=sscanf(argv[5], "%d", &y2);
	outname=argv[6];
	if(i!=4) myerror(-2, stderr, "Error reading corner locations.\n");

	i=ReadPPM(inname, &width1, &height1, &image1);
	if(i) myerror(-3, stderr, "Error reading input image file.\n");

	x1=mymax(0,x1);
	x1=mymin(width1-1,x1);
	y1=mymax(0,y1);
	y1=mymin(height1-1,y1);

	x2=mymax(0,x2);
	x2=mymin(width1-1,x2);
	y2=mymax(0,y2);
	y2=mymin(height1-1,y2);

	if(x2<=x1 || y2<=y1)
		myerror(-4, stderr, "Invalid corner specification.\n");

	width2=x2-x1+1;
	height2=y2-y1+1;

	image2=(uchar***)alloc3d(PPM_BANDS, height2, width2, sizeof(uchar));
	if(image2==NULL) myerror(-5, stderr, "Error allocating memory.\n");

	for(k=0;k<PPM_BANDS;k++)
		for(i=y1;i<=y2;i++)
			for(j=x1;j<=x2;j++)
				image2[k][i-y1][j-x1] = image1[k][i][j];

	time(&now);

	fp=fopen(outname, "w");
	if(fp==NULL) myerror(-6, stderr, "Error openning output file.\n");

	fprintf(fp, "P6\n");
	/* begin comments */
	fprintf(fp, "# Name: %s\n", outname);
	fprintf(fp, "# Produced by subppm on %s", ctime(&now));
	fprintf(fp, "# Cropped from %s\n", inname);
	fprintf(fp, "#   by taking the (%dx%d) region starting at (%d,%d)\n",
			width2, height2, x1, y1);
	/* end comments */
	fprintf(fp, "%d %d\n", width2, height2);
	fprintf(fp, "255\n");

	err=0;
	for(i=0;i<height2;i++)
		for(j=0;j<width2;j++)
			for(k=0;k<PPM_BANDS;k++) {
				c1=image2[k][i][j];
				c2=fputc(c1, fp);
				if(c1!=c2) err=1;
			}
	err |= fclose(fp);
	if(err) myerror(-4, stderr, "Error writing output.\n");

	free3d((void***)image1, PPM_BANDS, height1, width1);
	free3d((void***)image2, PPM_BANDS, height2, width2);

	return 0;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <PPM1> <x1> <y1> <x2> <y2> <PPM2>\n", cmd);
fprintf(stderr, "<PPM1> is the name of the input PPM file;\n");
fprintf(stderr, "<x1> <y1> left-top corner of the sub-image;\n");
fprintf(stderr, "<x2> <y2> right-bottom corner;\n");
fprintf(stderr, "<PPM2> is the name of the output PPM file.\n");
fprintf(stderr, "\n");
}
