/*
	File name:	thresholdpgm.c
	Type:		C main file.
	Author:		Gang Liu
	Date:		06/12/97
	Function:	Threshold an image in PGM file.
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "must.h"
#include "pgm.h"

void	usage(char *cmd);

int	main(int argc, char *argv[])
{
	char	op, verbose;
	float	T;
	uchar	newval;
	char	*fname;

	int	width, height;
	uchar	**image;

	int	i, j;
	long	count;

	if(argc<4) {
		usage(argv[0]);
		return -1;
	}

	verbose=0;
	if(argc>=5 && toupper(argv[4][0])=='V') verbose=1;

	T=atof(argv[2]);

	fname=argv[1];
	i=ReadPGM(fname, &width, &height, &image);
	if(i) myerror(-2, stderr, "Error reading input image.\n");

	count=0;
	for(i=0;i<height;i++)
		for(j=0;j<width;j++) {
			newval = ( (image[i][j] >= T) ? 255 : 0 );
			image[i][j] = newval;
			if(newval) count++;
		}

	fname=argv[3];
	i=WritePGM(fname, width, height, image);
	if(i) fprintf(stderr, "Error writing output.\n");

	if(verbose)
		printf("%d pixels with value greater than %f\n", count, T);

	free2d((void**)image);

	if(i) i=-2;	/* signaling error in writing output file */
	return i;
}

void	usage(char *cmd)
{
fprintf(stderr, "Usage:\n\t %s <PGM1> <threshold> <PGM2> [v]\n", cmd);
fprintf(stderr, "<PGM1> is input PGM file.\n");
fprintf(stderr, "<threshold> is threshold value.\n");
fprintf(stderr, "<PGM2> is output PGM file.\n");
fprintf(stderr, "[v]    verbose.\n");
fprintf(stderr, "\n");
return;
}
